@@ -1,10 +1,9 @@
-
-/* A Bison parser, made by GNU Bison 2.4.  */
+/* A Bison parser, made by GNU Bison 2.4.1.160-aa01.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+   2007, 2008, 2009 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +45,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.4"
+#define YYBISON_VERSION "2.4.1.160-aa01"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -67,7 +66,7 @@
 
 /* Copy the first part of user declarations.  */
 
-/* Line 189 of yacc.c  */
+/* Line 252 of yacc.c  */
 #line 1 "getdate.y"
 
 /* Parse a string into an internal time stamp.
@@ -149,16 +148,6 @@
    of `digit' even when the host does not conform to POSIX.  */
 #define ISDIGIT(c) ((unsigned int) (c) - '0' <= 9)
 
-#ifndef __attribute__
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 8) || __STRICT_ANSI__
-#  define __attribute__(x)
-# endif
-#endif
-
-#ifndef ATTRIBUTE_UNUSED
-# define ATTRIBUTE_UNUSED __attribute__ ((__unused__))
-#endif
-
 /* Shift A right by B bits portably, by dividing A by 2**B and
    truncating towards minus infinity.  A and B should be free of side
    effects, and B should be in the range 0 <= B <= INT_BITS - 2, where
@@ -179,12 +168,21 @@
 
 #define HOUR(x) ((x) * 60)
 
-/* Lots of this code assumes time_t and time_t-like values fit into
-   long int.  It also assumes that signed integer overflow silently
-   wraps around, but there's no portable way to check for that at
-   compile-time.  */
+/* long_time_t is a signed integer type that contains all time_t values.  */
 verify (TYPE_IS_INTEGER (time_t));
-verify (LONG_MIN <= TYPE_MINIMUM (time_t) && TYPE_MAXIMUM (time_t) <= LONG_MAX);
+#if TIME_T_FITS_IN_LONG_INT
+typedef long int long_time_t;
+#else
+typedef time_t long_time_t;
+#endif
+
+/* Lots of this code assumes time_t and time_t-like values fit into
+   long_time_t.  */
+verify (TYPE_MINIMUM (long_time_t) <= TYPE_MINIMUM (time_t)
+        && TYPE_MAXIMUM (time_t) <= TYPE_MAXIMUM (long_time_t));
+
+/* FIXME: It also assumes that signed integer overflow silently wraps around,
+   but this is not true any more with recent versions of GCC 4.  */
 
 /* An integer value, and the number of digits in its textual
    representation.  */
@@ -217,7 +215,7 @@ typedef struct
   long int day;
   long int hour;
   long int minutes;
-  long int seconds;
+  long_time_t seconds;
   long int ns;
 } relative_time;
 
@@ -344,8 +342,8 @@ set_hhmmss (parser_control *pc, long int hour, long int minutes,
 
 
 
-/* Line 189 of yacc.c  */
-#line 349 "getdate.c"
+/* Line 252 of yacc.c  */
+#line 347 "getdate.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -423,8 +421,8 @@ set_hhmmss (parser_control *pc, long int hour, long int minutes,
 typedef union YYSTYPE
 {
 
-/* Line 214 of yacc.c  */
-#line 286 "getdate.y"
+/* Line 277 of yacc.c  */
+#line 285 "getdate.y"
 
   long int intval;
   textint textintval;
@@ -433,8 +431,8 @@ typedef union YYSTYPE
 
 
 
-/* Line 214 of yacc.c  */
-#line 438 "getdate.c"
+/* Line 277 of yacc.c  */
+#line 436 "getdate.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -445,8 +443,8 @@ typedef union YYSTYPE
 /* Copy the second part of user declarations.  */
 
 
-/* Line 264 of yacc.c  */
-#line 450 "getdate.c"
+/* Line 327 of yacc.c  */
+#line 448 "getdate.c"
 
 #ifdef short
 # undef short
@@ -760,15 +758,15 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   312,   312,   313,   317,   324,   326,   330,   332,   334,
-     336,   338,   340,   341,   342,   346,   351,   356,   363,   368,
-     378,   383,   391,   393,   396,   398,   400,   405,   410,   415,
-     420,   428,   433,   453,   460,   468,   476,   481,   487,   492,
-     501,   503,   505,   510,   512,   514,   516,   518,   520,   522,
-     524,   526,   528,   530,   532,   534,   536,   538,   540,   542,
-     544,   546,   548,   550,   554,   556,   558,   560,   562,   564,
-     569,   573,   573,   576,   577,   582,   583,   588,   593,   604,
-     605,   611,   612
+       0,   311,   311,   312,   316,   323,   325,   329,   331,   333,
+     335,   337,   339,   340,   341,   345,   350,   355,   362,   367,
+     377,   382,   390,   392,   395,   397,   399,   404,   409,   414,
+     419,   427,   432,   452,   459,   467,   475,   480,   486,   491,
+     500,   502,   504,   509,   511,   513,   515,   517,   519,   521,
+     523,   525,   527,   529,   531,   533,   535,   537,   539,   541,
+     543,   545,   547,   549,   553,   555,   557,   559,   561,   563,
+     568,   572,   572,   575,   576,   581,   582,   587,   592,   603,
+     604,   610,   611
 };
 #endif
 
@@ -828,8 +826,8 @@ static const yytype_uint8 yyr2[] =
        2,     0,     1
 };
 
-/* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
-   STATE-NUM when YYTABLE doesn't specify something else to do.  Zero
+/* YYDEFACT[STATE-NAME] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE doesn't specify something else to do.  Zero
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
@@ -880,8 +878,7 @@ static const yytype_int8 yypgoto[] =
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
-   number is the opposite.  If zero, do what YYDEFACT says.
-   If YYTABLE_NINF, syntax error.  */
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
@@ -897,6 +894,12 @@ static const yytype_uint8 yytable[] =
       87,    88,    98,     0,     0,    96,     0,     0,    80
 };
 
+#define yypact_value_is_default(yystate) \
+  ((yystate) == (-82))
+
+#define yytable_value_is_error(yytable_value) \
+  YYID (0)
+
 static const yytype_int8 yycheck[] =
 {
        5,     6,     7,     8,     9,    10,    23,    12,    15,    90,
@@ -1304,111 +1307,121 @@ yytnamerr (char *yyres, const char *yystr)
 }
 # endif
 
-/* Copy into YYRESULT an error message about the unexpected token
-   YYCHAR while in state YYSTATE.  Return the number of bytes copied,
-   including the terminating null byte.  If YYRESULT is null, do not
-   copy anything; just return the number of bytes that would be
-   copied.  As a special case, return 0 if an ordinary "syntax error"
-   message will do.  Return YYSIZE_MAXIMUM if overflow occurs during
-   size calculation.  */
-static YYSIZE_T
-yysyntax_error (char *yyresult, int yystate, int yychar)
+/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
+   about the unexpected token YYTOKEN while in state YYSTATE.
+
+   Return 0 if *YYMSG was successfully written.  Return 1 if an ordinary
+   "syntax error" message will suffice instead.  Return 2 if *YYMSG is
+   not large enough to hold the message.  In the last case, also set
+   *YYMSG_ALLOC to either (a) the required number of bytes or (b) zero
+   if the required number of bytes is too large to store.  */
+static int
+yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
+                int yystate, int yytoken)
 {
   int yyn = yypact[yystate];
 
   if (! (YYPACT_NINF < yyn && yyn <= YYLAST))
-    return 0;
+    return 1;
   else
     {
-      int yytype = YYTRANSLATE (yychar);
-      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
+      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytoken]);
       YYSIZE_T yysize = yysize0;
       YYSIZE_T yysize1;
-      int yysize_overflow = 0;
       enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
+      /* Internationalized format string. */
+      const char *yyformat = 0;
+      /* Arguments of yyformat. */
       char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
-      int yyx;
-
-# if 0
-      /* This is so xgettext sees the translatable formats that are
-	 constructed on the fly.  */
-      YY_("syntax error, unexpected %s");
-      YY_("syntax error, unexpected %s, expecting %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
-# endif
-      char *yyfmt;
-      char const *yyf;
-      static char const yyunexpected[] = "syntax error, unexpected %s";
-      static char const yyexpecting[] = ", expecting %s";
-      static char const yyor[] = " or %s";
-      char yyformat[sizeof yyunexpected
-		    + sizeof yyexpecting - 1
-		    + ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
-		       * (sizeof yyor - 1))];
-      char const *yyprefix = yyexpecting;
 
       /* Start YYX at -YYN if negative to avoid negative indexes in
-	 YYCHECK.  */
+	 YYCHECK.  In other words, skip the first -YYN actions for this
+	 state because they are default actions.  */
       int yyxbegin = yyn < 0 ? -yyn : 0;
 
       /* Stay within bounds of both yycheck and yytname.  */
       int yychecklim = YYLAST - yyn + 1;
       int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-      int yycount = 1;
+      /* Number of reported tokens (one for the "unexpected", one per
+         "expected"). */
+      int yycount = 0;
+      int yyx;
 
-      yyarg[0] = yytname[yytype];
-      yyfmt = yystpcpy (yyformat, yyunexpected);
+      yyarg[yycount++] = yytname[yytoken];
 
       for (yyx = yyxbegin; yyx < yyxend; ++yyx)
-	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
+	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR
+	    && !yytable_value_is_error (yytable[yyx + yyn]))
 	  {
 	    if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
 	      {
 		yycount = 1;
 		yysize = yysize0;
-		yyformat[sizeof yyunexpected - 1] = '\0';
 		break;
 	      }
 	    yyarg[yycount++] = yytname[yyx];
 	    yysize1 = yysize + yytnamerr (0, yytname[yyx]);
-	    yysize_overflow |= (yysize1 < yysize);
+	    if (! (yysize <= yysize1
+		   && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+	      {
+		/* Overflow.  */
+		*yymsg_alloc = 0;
+		return 2;
+	      }
 	    yysize = yysize1;
-	    yyfmt = yystpcpy (yyfmt, yyprefix);
-	    yyprefix = yyor;
 	  }
 
-      yyf = YY_(yyformat);
-      yysize1 = yysize + yystrlen (yyf);
-      yysize_overflow |= (yysize1 < yysize);
+      switch (yycount)
+        {
+#define YYCASE_(N, S)                           \
+          case N:                               \
+            yyformat = S;                       \
+          break
+          YYCASE_(1, YY_("syntax error, unexpected %s"));
+          YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
+          YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
+          YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
+          YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
+#undef YYCASE_
+        }
+
+      yysize1 = yysize + yystrlen (yyformat);
+      if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+        {
+          /* Overflow.  */
+          *yymsg_alloc = 0;
+          return 2;
+        }
       yysize = yysize1;
 
-      if (yysize_overflow)
-	return YYSIZE_MAXIMUM;
+      if (*yymsg_alloc < yysize)
+        {
+          *yymsg_alloc = 2 * yysize;
+          if (! (yysize <= *yymsg_alloc
+                 && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
+            *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
+          return 2;
+        }
 
-      if (yyresult)
-	{
-	  /* Avoid sprintf, as that infringes on the user's name space.
-	     Don't have undefined behavior even if the translation
-	     produced a string with the wrong number of "%s"s.  */
-	  char *yyp = yyresult;
-	  int yyi = 0;
-	  while ((*yyp = *yyf) != '\0')
-	    {
-	      if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
-		{
-		  yyp += yytnamerr (yyp, yyarg[yyi++]);
-		  yyf += 2;
-		}
-	      else
-		{
-		  yyp++;
-		  yyf++;
-		}
-	    }
-	}
-      return yysize;
+      /* Avoid sprintf, as that infringes on the user's name space.
+         Don't have undefined behavior even if the translation
+         produced a string with the wrong number of "%s"s.  */
+      {
+        char *yyp = *yymsg;
+        int yyi = 0;
+        while ((*yyp = *yyformat) != '\0')
+          if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
+            {
+              yyp += yytnamerr (yyp, yyarg[yyi++]);
+              yyformat += 2;
+            }
+          else
+            {
+              yyp++;
+              yyformat++;
+            }
+      }
+      return 0;
     }
 }
 #endif /* YYERROR_VERBOSE */
@@ -1447,6 +1460,7 @@ yydestruct (yymsg, yytype, yyvaluep, pc)
     }
 }
 
+
 /* Prevent warnings from -Wmissing-prototypes.  */
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
@@ -1463,12 +1477,9 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-
-
-/*-------------------------.
-| yyparse or yypush_parse.  |
-`-------------------------*/
+/*----------.
+| yyparse.  |
+`----------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1655,7 +1666,7 @@ yybackup:
 
   /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
-  if (yyn == YYPACT_NINF)
+  if (yypact_value_is_default (yyn))
     goto yydefault;
 
   /* Not known => get a lookahead token if don't already have one.  */
@@ -1686,7 +1697,7 @@ yybackup:
   yyn = yytable[yyn];
   if (yyn <= 0)
     {
-      if (yyn == 0 || yyn == YYTABLE_NINF)
+      if (yytable_value_is_error (yyn))
 	goto yyerrlab;
       yyn = -yyn;
       goto yyreduce;
@@ -1742,8 +1753,8 @@ yyreduce:
     {
         case 4:
 
-/* Line 1455 of yacc.c  */
-#line 318 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 317 "getdate.y"
     {
 	pc->seconds = (yyvsp[(2) - (2)].timespec);
 	pc->timespec_seen = true;
@@ -1752,43 +1763,43 @@ yyreduce:
 
   case 7:
 
-/* Line 1455 of yacc.c  */
-#line 331 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 330 "getdate.y"
     { pc->times_seen++; }
     break;
 
   case 8:
 
-/* Line 1455 of yacc.c  */
-#line 333 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 332 "getdate.y"
     { pc->local_zones_seen++; }
     break;
 
   case 9:
 
-/* Line 1455 of yacc.c  */
-#line 335 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 334 "getdate.y"
     { pc->zones_seen++; }
     break;
 
   case 10:
 
-/* Line 1455 of yacc.c  */
-#line 337 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 336 "getdate.y"
     { pc->dates_seen++; }
     break;
 
   case 11:
 
-/* Line 1455 of yacc.c  */
-#line 339 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 338 "getdate.y"
     { pc->days_seen++; }
     break;
 
   case 15:
 
-/* Line 1455 of yacc.c  */
-#line 347 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 346 "getdate.y"
     {
 	set_hhmmss (pc, (yyvsp[(1) - (2)].textintval).value, 0, 0, 0);
 	pc->meridian = (yyvsp[(2) - (2)].intval);
@@ -1797,8 +1808,8 @@ yyreduce:
 
   case 16:
 
-/* Line 1455 of yacc.c  */
-#line 352 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 351 "getdate.y"
     {
 	set_hhmmss (pc, (yyvsp[(1) - (4)].textintval).value, (yyvsp[(3) - (4)].textintval).value, 0, 0);
 	pc->meridian = (yyvsp[(4) - (4)].intval);
@@ -1807,8 +1818,8 @@ yyreduce:
 
   case 17:
 
-/* Line 1455 of yacc.c  */
-#line 357 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 356 "getdate.y"
     {
 	set_hhmmss (pc, (yyvsp[(1) - (5)].textintval).value, (yyvsp[(3) - (5)].textintval).value, 0, 0);
 	pc->meridian = MER24;
@@ -1819,8 +1830,8 @@ yyreduce:
 
   case 18:
 
-/* Line 1455 of yacc.c  */
-#line 364 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 363 "getdate.y"
     {
 	set_hhmmss (pc, (yyvsp[(1) - (6)].textintval).value, (yyvsp[(3) - (6)].textintval).value, (yyvsp[(5) - (6)].timespec).tv_sec, (yyvsp[(5) - (6)].timespec).tv_nsec);
 	pc->meridian = (yyvsp[(6) - (6)].intval);
@@ -1829,8 +1840,8 @@ yyreduce:
 
   case 19:
 
-/* Line 1455 of yacc.c  */
-#line 369 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 368 "getdate.y"
     {
 	set_hhmmss (pc, (yyvsp[(1) - (7)].textintval).value, (yyvsp[(3) - (7)].textintval).value, (yyvsp[(5) - (7)].timespec).tv_sec, (yyvsp[(5) - (7)].timespec).tv_nsec);
 	pc->meridian = MER24;
@@ -1841,8 +1852,8 @@ yyreduce:
 
   case 20:
 
-/* Line 1455 of yacc.c  */
-#line 379 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 378 "getdate.y"
     {
 	pc->local_isdst = (yyvsp[(1) - (1)].intval);
 	pc->dsts_seen += (0 < (yyvsp[(1) - (1)].intval));
@@ -1851,8 +1862,8 @@ yyreduce:
 
   case 21:
 
-/* Line 1455 of yacc.c  */
-#line 384 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 383 "getdate.y"
     {
 	pc->local_isdst = 1;
 	pc->dsts_seen += (0 < (yyvsp[(1) - (2)].intval)) + 1;
@@ -1861,44 +1872,44 @@ yyreduce:
 
   case 22:
 
-/* Line 1455 of yacc.c  */
-#line 392 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 391 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (1)].intval); }
     break;
 
   case 23:
 
-/* Line 1455 of yacc.c  */
-#line 394 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 393 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (2)].intval);
 	apply_relative_time (pc, (yyvsp[(2) - (2)].rel), 1); }
     break;
 
   case 24:
 
-/* Line 1455 of yacc.c  */
-#line 397 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 396 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (3)].intval) + time_zone_hhmm (pc, (yyvsp[(2) - (3)].textintval), (yyvsp[(3) - (3)].intval)); }
     break;
 
   case 25:
 
-/* Line 1455 of yacc.c  */
-#line 399 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 398 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (1)].intval) + 60; }
     break;
 
   case 26:
 
-/* Line 1455 of yacc.c  */
-#line 401 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 400 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (2)].intval) + 60; }
     break;
 
   case 27:
 
-/* Line 1455 of yacc.c  */
-#line 406 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 405 "getdate.y"
     {
 	pc->day_ordinal = 0;
 	pc->day_number = (yyvsp[(1) - (1)].intval);
@@ -1907,8 +1918,8 @@ yyreduce:
 
   case 28:
 
-/* Line 1455 of yacc.c  */
-#line 411 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 410 "getdate.y"
     {
 	pc->day_ordinal = 0;
 	pc->day_number = (yyvsp[(1) - (2)].intval);
@@ -1917,8 +1928,8 @@ yyreduce:
 
   case 29:
 
-/* Line 1455 of yacc.c  */
-#line 416 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 415 "getdate.y"
     {
 	pc->day_ordinal = (yyvsp[(1) - (2)].intval);
 	pc->day_number = (yyvsp[(2) - (2)].intval);
@@ -1927,8 +1938,8 @@ yyreduce:
 
   case 30:
 
-/* Line 1455 of yacc.c  */
-#line 421 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 420 "getdate.y"
     {
 	pc->day_ordinal = (yyvsp[(1) - (2)].textintval).value;
 	pc->day_number = (yyvsp[(2) - (2)].intval);
@@ -1937,8 +1948,8 @@ yyreduce:
 
   case 31:
 
-/* Line 1455 of yacc.c  */
-#line 429 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 428 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (3)].textintval).value;
 	pc->day = (yyvsp[(3) - (3)].textintval).value;
@@ -1947,8 +1958,8 @@ yyreduce:
 
   case 32:
 
-/* Line 1455 of yacc.c  */
-#line 434 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 433 "getdate.y"
     {
 	/* Interpret as YYYY/MM/DD if the first value has 4 or more digits,
 	   otherwise as MM/DD/YY.
@@ -1972,8 +1983,8 @@ yyreduce:
 
   case 33:
 
-/* Line 1455 of yacc.c  */
-#line 454 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 453 "getdate.y"
     {
 	/* ISO 8601 format.  YYYY-MM-DD.  */
 	pc->year = (yyvsp[(1) - (3)].textintval);
@@ -1984,8 +1995,8 @@ yyreduce:
 
   case 34:
 
-/* Line 1455 of yacc.c  */
-#line 461 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 460 "getdate.y"
     {
 	/* e.g. 17-JUN-1992.  */
 	pc->day = (yyvsp[(1) - (3)].textintval).value;
@@ -1997,8 +2008,8 @@ yyreduce:
 
   case 35:
 
-/* Line 1455 of yacc.c  */
-#line 469 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 468 "getdate.y"
     {
 	/* e.g. JUN-17-1992.  */
 	pc->month = (yyvsp[(1) - (3)].intval);
@@ -2010,8 +2021,8 @@ yyreduce:
 
   case 36:
 
-/* Line 1455 of yacc.c  */
-#line 477 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 476 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (2)].intval);
 	pc->day = (yyvsp[(2) - (2)].textintval).value;
@@ -2020,8 +2031,8 @@ yyreduce:
 
   case 37:
 
-/* Line 1455 of yacc.c  */
-#line 482 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 481 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (4)].intval);
 	pc->day = (yyvsp[(2) - (4)].textintval).value;
@@ -2031,8 +2042,8 @@ yyreduce:
 
   case 38:
 
-/* Line 1455 of yacc.c  */
-#line 488 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 487 "getdate.y"
     {
 	pc->day = (yyvsp[(1) - (2)].textintval).value;
 	pc->month = (yyvsp[(2) - (2)].intval);
@@ -2041,8 +2052,8 @@ yyreduce:
 
   case 39:
 
-/* Line 1455 of yacc.c  */
-#line 493 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 492 "getdate.y"
     {
 	pc->day = (yyvsp[(1) - (3)].textintval).value;
 	pc->month = (yyvsp[(2) - (3)].intval);
@@ -2052,239 +2063,239 @@ yyreduce:
 
   case 40:
 
-/* Line 1455 of yacc.c  */
-#line 502 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 501 "getdate.y"
     { apply_relative_time (pc, (yyvsp[(1) - (2)].rel), -1); }
     break;
 
   case 41:
 
-/* Line 1455 of yacc.c  */
-#line 504 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 503 "getdate.y"
     { apply_relative_time (pc, (yyvsp[(1) - (1)].rel), 1); }
     break;
 
   case 42:
 
-/* Line 1455 of yacc.c  */
-#line 506 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 505 "getdate.y"
     { apply_relative_time (pc, (yyvsp[(1) - (1)].rel), 1); }
     break;
 
   case 43:
 
-/* Line 1455 of yacc.c  */
-#line 511 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 510 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 44:
 
-/* Line 1455 of yacc.c  */
-#line 513 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 512 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 45:
 
-/* Line 1455 of yacc.c  */
-#line 515 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 514 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = 1; }
     break;
 
   case 46:
 
-/* Line 1455 of yacc.c  */
-#line 517 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 516 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 47:
 
-/* Line 1455 of yacc.c  */
-#line 519 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 518 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 48:
 
-/* Line 1455 of yacc.c  */
-#line 521 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 520 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = 1; }
     break;
 
   case 49:
 
-/* Line 1455 of yacc.c  */
-#line 523 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 522 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].intval) * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 50:
 
-/* Line 1455 of yacc.c  */
-#line 525 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 524 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].textintval).value * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 51:
 
-/* Line 1455 of yacc.c  */
-#line 527 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 526 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (1)].intval); }
     break;
 
   case 52:
 
-/* Line 1455 of yacc.c  */
-#line 529 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 528 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 53:
 
-/* Line 1455 of yacc.c  */
-#line 531 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 530 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 54:
 
-/* Line 1455 of yacc.c  */
-#line 533 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 532 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = 1; }
     break;
 
   case 55:
 
-/* Line 1455 of yacc.c  */
-#line 535 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 534 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 56:
 
-/* Line 1455 of yacc.c  */
-#line 537 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 536 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 57:
 
-/* Line 1455 of yacc.c  */
-#line 539 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 538 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = 1; }
     break;
 
   case 58:
 
-/* Line 1455 of yacc.c  */
-#line 541 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 540 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 59:
 
-/* Line 1455 of yacc.c  */
-#line 543 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 542 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 60:
 
-/* Line 1455 of yacc.c  */
-#line 545 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 544 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].timespec).tv_sec; (yyval.rel).ns = (yyvsp[(1) - (2)].timespec).tv_nsec; }
     break;
 
   case 61:
 
-/* Line 1455 of yacc.c  */
-#line 547 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 546 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].timespec).tv_sec; (yyval.rel).ns = (yyvsp[(1) - (2)].timespec).tv_nsec; }
     break;
 
   case 62:
 
-/* Line 1455 of yacc.c  */
-#line 549 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 548 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = 1; }
     break;
 
   case 64:
 
-/* Line 1455 of yacc.c  */
-#line 555 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 554 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 65:
 
-/* Line 1455 of yacc.c  */
-#line 557 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 556 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 66:
 
-/* Line 1455 of yacc.c  */
-#line 559 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 558 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].textintval).value * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 67:
 
-/* Line 1455 of yacc.c  */
-#line 561 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 560 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 68:
 
-/* Line 1455 of yacc.c  */
-#line 563 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 562 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 69:
 
-/* Line 1455 of yacc.c  */
-#line 565 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 564 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 70:
 
-/* Line 1455 of yacc.c  */
-#line 570 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 569 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (1)].intval); }
     break;
 
   case 74:
 
-/* Line 1455 of yacc.c  */
-#line 578 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 577 "getdate.y"
     { (yyval.timespec).tv_sec = (yyvsp[(1) - (1)].textintval).value; (yyval.timespec).tv_nsec = 0; }
     break;
 
   case 76:
 
-/* Line 1455 of yacc.c  */
-#line 584 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 583 "getdate.y"
     { (yyval.timespec).tv_sec = (yyvsp[(1) - (1)].textintval).value; (yyval.timespec).tv_nsec = 0; }
     break;
 
   case 77:
 
-/* Line 1455 of yacc.c  */
-#line 589 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 588 "getdate.y"
     { digits_to_date_time (pc, (yyvsp[(1) - (1)].textintval)); }
     break;
 
   case 78:
 
-/* Line 1455 of yacc.c  */
-#line 594 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 593 "getdate.y"
     {
 	/* Hybrid all-digit and relative offset, so that we accept e.g.,
 	   "YYYYMMDD +N days" as well as "YYYYMMDD N days".  */
@@ -2295,36 +2306,36 @@ yyreduce:
 
   case 79:
 
-/* Line 1455 of yacc.c  */
-#line 604 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 603 "getdate.y"
     { (yyval.intval) = -1; }
     break;
 
   case 80:
 
-/* Line 1455 of yacc.c  */
-#line 606 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 605 "getdate.y"
     { (yyval.intval) = (yyvsp[(2) - (2)].textintval).value; }
     break;
 
   case 81:
 
-/* Line 1455 of yacc.c  */
-#line 611 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 610 "getdate.y"
     { (yyval.intval) = MER24; }
     break;
 
   case 82:
 
-/* Line 1455 of yacc.c  */
-#line 613 "getdate.y"
+/* Line 1479 of yacc.c  */
+#line 612 "getdate.y"
     { (yyval.intval) = (yyvsp[(1) - (1)].intval); }
     break;
 
 
 
-/* Line 1455 of yacc.c  */
-#line 2328 "getdate.c"
+/* Line 1479 of yacc.c  */
+#line 2339 "getdate.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2361,37 +2372,28 @@ yyerrlab:
 #if ! YYERROR_VERBOSE
       yyerror (pc, YY_("syntax error"));
 #else
-      {
-	YYSIZE_T yysize = yysyntax_error (0, yystate, yychar);
-	if (yymsg_alloc < yysize && yymsg_alloc < YYSTACK_ALLOC_MAXIMUM)
-	  {
-	    YYSIZE_T yyalloc = 2 * yysize;
-	    if (! (yysize <= yyalloc && yyalloc <= YYSTACK_ALLOC_MAXIMUM))
-	      yyalloc = YYSTACK_ALLOC_MAXIMUM;
-	    if (yymsg != yymsgbuf)
-	      YYSTACK_FREE (yymsg);
-	    yymsg = (char *) YYSTACK_ALLOC (yyalloc);
-	    if (yymsg)
-	      yymsg_alloc = yyalloc;
-	    else
-	      {
-		yymsg = yymsgbuf;
-		yymsg_alloc = sizeof yymsgbuf;
-	      }
-	  }
-
-	if (0 < yysize && yysize <= yymsg_alloc)
-	  {
-	    (void) yysyntax_error (yymsg, yystate, yychar);
-	    yyerror (pc, yymsg);
-	  }
-	else
-	  {
-	    yyerror (pc, YY_("syntax error"));
-	    if (yysize != 0)
-	      goto yyexhaustedlab;
-	  }
-      }
+      while (1)
+        {
+          int yysyntax_error_status =
+            yysyntax_error (&yymsg_alloc, &yymsg, yystate, yytoken);
+          if (yysyntax_error_status == 2 && 0 < yymsg_alloc)
+            {
+              if (yymsg != yymsgbuf)
+                YYSTACK_FREE (yymsg);
+              yymsg = (char *) YYSTACK_ALLOC (yymsg_alloc);
+              if (yymsg)
+                continue;
+              yymsg = yymsgbuf;
+              yymsg_alloc = sizeof yymsgbuf;
+            }
+          if (yysyntax_error_status == 0)
+            yyerror (pc, yymsg);
+          else
+            yyerror (pc, YY_("syntax error"));
+          if (yysyntax_error_status == 2)
+            goto yyexhaustedlab;
+          break;
+        }
 #endif
     }
 
@@ -2450,7 +2452,7 @@ yyerrlab1:
   for (;;)
     {
       yyn = yypact[yystate];
-      if (yyn != YYPACT_NINF)
+      if (!yypact_value_is_default (yyn))
 	{
 	  yyn += YYTERROR;
 	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
@@ -2535,8 +2537,8 @@ yyreturn:
 
 
 
-/* Line 1675 of yacc.c  */
-#line 616 "getdate.y"
+/* Line 1690 of yacc.c  */
+#line 615 "getdate.y"
 
 
 static table const meridian_table[] =
@@ -3070,8 +3072,8 @@ yylex (YYSTYPE *lvalp, parser_control *pc)
 
 /* Do nothing if the parser reports an error.  */
 static int
-yyerror (parser_control const *pc ATTRIBUTE_UNUSED,
-	 char const *s ATTRIBUTE_UNUSED)
+yyerror (parser_control const *pc _UNUSED_PARAMETER_,
+	 char const *s _UNUSED_PARAMETER_)
 {
   return 0;
 }
@@ -3426,20 +3428,22 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
 	time_t t1 = t0 + d1;
 	long int d2 = 60 * pc.rel.minutes;
 	time_t t2 = t1 + d2;
-	long int d3 = pc.rel.seconds;
-	time_t t3 = t2 + d3;
+	long_time_t d3 = pc.rel.seconds;
+	long_time_t t3 = t2 + d3;
 	long int d4 = (sum_ns - normalized_ns) / BILLION;
-	time_t t4 = t3 + d4;
+	long_time_t t4 = t3 + d4;
+	time_t t5 = t4;
 
 	if ((d1 / (60 * 60) ^ pc.rel.hour)
 	    | (d2 / 60 ^ pc.rel.minutes)
 	    | ((t1 < t0) ^ (d1 < 0))
 	    | ((t2 < t1) ^ (d2 < 0))
 	    | ((t3 < t2) ^ (d3 < 0))
-	    | ((t4 < t3) ^ (d4 < 0)))
+	    | ((t4 < t3) ^ (d4 < 0))
+	    | (t5 != t4))
 	  goto fail;
 
-	result->tv_sec = t4;
+	result->tv_sec = t5;
 	result->tv_nsec = normalized_ns;
       }
     }
