@@ -1,5 +1,5 @@
 /* nohup -- run a command immune to hangups, with output to a non-tty
-   Copyright (C) 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2007-2009 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,30 +34,28 @@
 
 #define PROGRAM_NAME "nohup"
 
-#define AUTHORS "Jim Meyering"
+#define AUTHORS proper_name ("Jim Meyering")
 
 /* Exit statuses.  */
 enum
   {
     /* `nohup' itself failed.  */
-    NOHUP_FAILURE = 127
+    POSIX_NOHUP_FAILURE = 127
   };
 
-char *program_name;
-
 void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
 Usage: %s COMMAND [ARG]...\n\
   or:  %s OPTION\n\
 "),
-	      program_name, program_name);
+              program_name, program_name);
 
       fputs (_("\
 Run COMMAND, ignoring hangup signals.\n\
@@ -71,9 +69,9 @@ If standard output is a terminal, append output to `nohup.out' if possible,\n\
 `$HOME/nohup.out' otherwise.\n\
 If standard error is a terminal, redirect it to standard output.\n\
 To save output to FILE, use `%s COMMAND > FILE'.\n"),
-	      program_name);
+              program_name);
       printf (USAGE_BUILTIN_WARNING, PROGRAM_NAME);
-      emit_bug_reporting_address ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -87,25 +85,32 @@ main (int argc, char **argv)
   bool redirecting_stdout;
   bool stdout_is_closed;
   bool redirecting_stderr;
+  int exit_internal_failure;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  initialize_exit_failure (NOHUP_FAILURE);
+  /* POSIX 2008 requires that internal failure give status 127; unlike
+     for env, exec, nice, time, and xargs where it requires internal
+     failure give something in the range 1-125.  For consistency with
+     other tools, fail with EXIT_CANCELED unless POSIXLY_CORRECT.  */
+  exit_internal_failure = (getenv ("POSIXLY_CORRECT")
+                           ? POSIX_NOHUP_FAILURE : EXIT_CANCELED);
+  initialize_exit_failure (exit_internal_failure);
   atexit (close_stdout);
 
-  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, VERSION,
-		      usage, AUTHORS, (char const *) NULL);
+  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, Version,
+                      usage, AUTHORS, (char const *) NULL);
   if (getopt_long (argc, argv, "+", NULL, NULL) != -1)
-    usage (NOHUP_FAILURE);
+    usage (exit_internal_failure);
 
   if (argc <= optind)
     {
       error (0, 0, _("missing operand"));
-      usage (NOHUP_FAILURE);
+      usage (exit_internal_failure);
     }
 
   ignoring_input = isatty (STDIN_FILENO);
@@ -120,7 +125,7 @@ main (int argc, char **argv)
     {
       fd_reopen (STDIN_FILENO, "/dev/null", O_WRONLY, 0);
       if (!redirecting_stdout && !redirecting_stderr)
-	error (0, 0, _("ignoring input"));
+        error (0, 0, _("ignoring input"));
     }
 
   /* If standard output is a tty, redirect it (appending) to a file.
@@ -135,38 +140,38 @@ main (int argc, char **argv)
       mode_t mode = S_IRUSR | S_IWUSR;
       mode_t umask_value = umask (~mode);
       out_fd = (redirecting_stdout
-		? fd_reopen (STDOUT_FILENO, file, flags, mode)
-		: open (file, flags, mode));
+                ? fd_reopen (STDOUT_FILENO, file, flags, mode)
+                : open (file, flags, mode));
 
       if (out_fd < 0)
-	{
-	  int saved_errno = errno;
-	  char const *home = getenv ("HOME");
-	  if (home)
-	    {
-	      in_home = file_name_concat (home, file, NULL);
-	      out_fd = (redirecting_stdout
-			? fd_reopen (STDOUT_FILENO, in_home, flags, mode)
-			: open (in_home, flags, mode));
-	    }
-	  if (out_fd < 0)
-	    {
-	      int saved_errno2 = errno;
-	      error (0, saved_errno, _("failed to open %s"), quote (file));
-	      if (in_home)
-		error (0, saved_errno2, _("failed to open %s"),
-		       quote (in_home));
-	      exit (NOHUP_FAILURE);
-	    }
-	  file = in_home;
-	}
+        {
+          int saved_errno = errno;
+          char const *home = getenv ("HOME");
+          if (home)
+            {
+              in_home = file_name_concat (home, file, NULL);
+              out_fd = (redirecting_stdout
+                        ? fd_reopen (STDOUT_FILENO, in_home, flags, mode)
+                        : open (in_home, flags, mode));
+            }
+          if (out_fd < 0)
+            {
+              int saved_errno2 = errno;
+              error (0, saved_errno, _("failed to open %s"), quote (file));
+              if (in_home)
+                error (0, saved_errno2, _("failed to open %s"),
+                       quote (in_home));
+              exit (exit_internal_failure);
+            }
+          file = in_home;
+        }
 
       umask (umask_value);
       error (0, 0,
-	     _(ignoring_input
-	       ? "ignoring input and appending output to %s"
-	       : "appending output to %s"),
-	     quote (file));
+             _(ignoring_input
+               ? N_("ignoring input and appending output to %s")
+               : N_("appending output to %s")),
+             quote (file));
       free (in_home);
     }
 
@@ -174,29 +179,39 @@ main (int argc, char **argv)
   if (redirecting_stderr)
     {
       /* Save a copy of stderr before redirecting, so we can use the original
-	 if execve fails.  It's no big deal if this dup fails.  It might
-	 not change anything, and at worst, it'll lead to suppression of
-	 the post-failed-execve diagnostic.  */
+         if execve fails.  It's no big deal if this dup fails.  It might
+         not change anything, and at worst, it'll lead to suppression of
+         the post-failed-execve diagnostic.  */
       saved_stderr_fd = dup (STDERR_FILENO);
 
       if (0 <= saved_stderr_fd
-	  && set_cloexec_flag (saved_stderr_fd, true) != 0)
-	error (NOHUP_FAILURE, errno,
-	       _("failed to set the copy of stderr to close on exec"));
+          && set_cloexec_flag (saved_stderr_fd, true) != 0)
+        error (exit_internal_failure, errno,
+               _("failed to set the copy of stderr to close on exec"));
 
       if (!redirecting_stdout)
-	error (0, 0,
-	       _(ignoring_input
-		 ? "ignoring input and redirecting stderr to stdout"
-		 : "redirecting stderr to stdout"));
+        error (0, 0,
+               _(ignoring_input
+                 ? N_("ignoring input and redirecting stderr to stdout")
+                 : N_("redirecting stderr to stdout")));
 
       if (dup2 (out_fd, STDERR_FILENO) < 0)
-	error (NOHUP_FAILURE, errno, _("failed to redirect standard error"));
+        error (exit_internal_failure, errno,
+               _("failed to redirect standard error"));
 
       if (stdout_is_closed)
-	close (out_fd);
+        close (out_fd);
     }
 
+  /* error() flushes stderr, but does not check for write failure.
+     Normally, we would catch this via our atexit() hook of
+     close_stdout, but execvp() gets in the way.  If stderr
+     encountered a write failure, there is no need to try calling
+     error() again, particularly since we may have just changed the
+     underlying fd out from under stderr.  */
+  if (ferror (stderr))
+    exit (exit_internal_failure);
+
   signal (SIGHUP, SIG_IGN);
 
   {
@@ -211,11 +226,11 @@ main (int argc, char **argv)
     /* The execve failed.  Output a diagnostic to stderr only if:
        - stderr was initially redirected to a non-tty, or
        - stderr was initially directed to a tty, and we
-	 can dup2 it to point back to that same tty.
+         can dup2 it to point back to that same tty.
        In other words, output the diagnostic if possible, but only if
        it will go to the original stderr.  */
     if (dup2 (saved_stderr_fd, STDERR_FILENO) == STDERR_FILENO)
-      error (0, saved_errno, _("cannot run command %s"), quote (*cmd));
+      error (0, saved_errno, _("failed to run command %s"), quote (*cmd));
 
     exit (exit_status);
   }
