@@ -1,5 +1,5 @@
 /* uniq -- remove duplicate lines from a sorted file
-   Copyright (C) 86, 91, 1995-2008 Free Software Foundation, Inc.
+   Copyright (C) 86, 91, 1995-2009 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 #include <config.h>
 
-#include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 
@@ -26,8 +25,10 @@
 #include "argmatch.h"
 #include "linebuffer.h"
 #include "error.h"
+#include "hard-locale.h"
 #include "posixver.h"
 #include "quote.h"
+#include "stdio--.h"
 #include "xmemcoll.h"
 #include "xstrtol.h"
 #include "memcasecmp.h"
@@ -127,16 +128,18 @@ usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
 Usage: %s [OPTION]... [INPUT [OUTPUT]]\n\
 "),
-	      program_name);
+              program_name);
       fputs (_("\
-Discard all but one of successive identical lines from INPUT (or\n\
-standard input), writing to OUTPUT (or standard output).\n\
+Filter adjacent matching lines from INPUT (or standard input),\n\
+writing to OUTPUT (or standard output).\n\
+\n\
+With no options, matching lines are merged to the first occurrence.\n\
 \n\
 "), stdout);
      fputs (_("\
@@ -170,8 +173,9 @@ characters.  Fields are skipped before chars.\n\
 \n\
 Note: 'uniq' does not detect repeated lines unless they are adjacent.\n\
 You may want to sort the input first, or use `sort -u' without `uniq'.\n\
+Also, comparisons honor the rules specified by `LC_COLLATE'.\n\
 "), stdout);
-      emit_bug_reporting_address ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -212,9 +216,9 @@ find_field (struct linebuffer const *line)
   for (count = 0; count < skip_fields; count++)
     {
       while (i < size && isblank (to_uchar (lp[i])))
-	i++;
+        i++;
       while (i < size && !isblank (to_uchar (lp[i])))
-	i++;
+        i++;
     }
 
   for (count = 0; count < skip_chars && i < size; count++)
@@ -255,11 +259,11 @@ different (char *old, char *new, size_t oldlen, size_t newlen)
 
 static void
 writeline (struct linebuffer const *line,
-	   bool match, uintmax_t linecount)
+           bool match, uintmax_t linecount)
 {
   if (! (linecount == 0 ? output_unique
-	 : !match ? output_first_repeated
-	 : output_later_repeated))
+         : !match ? output_first_repeated
+         : output_later_repeated))
     return;
 
   if (countmode == count_occurrences)
@@ -301,24 +305,24 @@ check_file (const char *infile, const char *outfile, char delimiter)
       size_t prevlen IF_LINT (= 0);
 
       while (!feof (stdin))
-	{
-	  char *thisfield;
-	  size_t thislen;
-	  if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
-	    break;
-	  thisfield = find_field (thisline);
-	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
-	  if (prevline->length == 0
-	      || different (thisfield, prevfield, thislen, prevlen))
-	    {
-	      fwrite (thisline->buffer, sizeof (char),
-		      thisline->length, stdout);
+        {
+          char *thisfield;
+          size_t thislen;
+          if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
+            break;
+          thisfield = find_field (thisline);
+          thislen = thisline->length - 1 - (thisfield - thisline->buffer);
+          if (prevline->length == 0
+              || different (thisfield, prevfield, thislen, prevlen))
+            {
+              fwrite (thisline->buffer, sizeof (char),
+                      thisline->length, stdout);
 
-	      SWAP_LINES (prevline, thisline);
-	      prevfield = thisfield;
-	      prevlen = thislen;
-	    }
-	}
+              SWAP_LINES (prevline, thisline);
+              prevfield = thisfield;
+              prevlen = thislen;
+            }
+        }
     }
   else
     {
@@ -328,59 +332,59 @@ check_file (const char *infile, const char *outfile, char delimiter)
       bool first_delimiter = true;
 
       if (readlinebuffer_delim (prevline, stdin, delimiter) == 0)
-	goto closefiles;
+        goto closefiles;
       prevfield = find_field (prevline);
       prevlen = prevline->length - 1 - (prevfield - prevline->buffer);
 
       while (!feof (stdin))
-	{
-	  bool match;
-	  char *thisfield;
-	  size_t thislen;
-	  if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
-	    {
-	      if (ferror (stdin))
-		goto closefiles;
-	      break;
-	    }
-	  thisfield = find_field (thisline);
-	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
-	  match = !different (thisfield, prevfield, thislen, prevlen);
-	  match_count += match;
+        {
+          bool match;
+          char *thisfield;
+          size_t thislen;
+          if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
+            {
+              if (ferror (stdin))
+                goto closefiles;
+              break;
+            }
+          thisfield = find_field (thisline);
+          thislen = thisline->length - 1 - (thisfield - thisline->buffer);
+          match = !different (thisfield, prevfield, thislen, prevlen);
+          match_count += match;
 
-	  if (match_count == UINTMAX_MAX)
-	    {
-	      if (count_occurrences)
-		error (EXIT_FAILURE, 0, _("too many repeated lines"));
-	      match_count--;
-	    }
+          if (match_count == UINTMAX_MAX)
+            {
+              if (count_occurrences)
+                error (EXIT_FAILURE, 0, _("too many repeated lines"));
+              match_count--;
+            }
 
           if (delimit_groups != DM_NONE)
-	    {
-	      if (!match)
-		{
-		  if (match_count) /* a previous match */
-		    first_delimiter = false; /* Only used when DM_SEPARATE */
-		}
-	      else if (match_count == 1)
-		{
-		  if ((delimit_groups == DM_PREPEND)
-		      || (delimit_groups == DM_SEPARATE
-			  && !first_delimiter))
-		    putchar (delimiter);
-		}
-	    }
+            {
+              if (!match)
+                {
+                  if (match_count) /* a previous match */
+                    first_delimiter = false; /* Only used when DM_SEPARATE */
+                }
+              else if (match_count == 1)
+                {
+                  if ((delimit_groups == DM_PREPEND)
+                      || (delimit_groups == DM_SEPARATE
+                          && !first_delimiter))
+                    putchar (delimiter);
+                }
+            }
 
-	  if (!match || output_later_repeated)
-	    {
-	      writeline (prevline, match, match_count);
-	      SWAP_LINES (prevline, thisline);
-	      prevfield = thisfield;
-	      prevlen = thislen;
-	      if (!match)
-		match_count = 0;
-	    }
-	}
+          if (!match || output_later_repeated)
+            {
+              writeline (prevline, match, match_count);
+              SWAP_LINES (prevline, thisline);
+              prevfield = thisfield;
+              prevlen = thislen;
+              if (!match)
+                match_count = 0;
+            }
+        }
 
       writeline (prevline, false, match_count);
     }
@@ -437,121 +441,121 @@ main (int argc, char **argv)
          obsolete.  */
 
       if (optc == -1
-	  || (posixly_correct && nfiles != 0)
-	  || ((optc = getopt_long (argc, argv,
-				   "-0123456789Dcdf:is:uw:z", longopts, NULL))
-	      == -1))
-	{
-	  if (argc <= optind)
-	    break;
-	  if (nfiles == 2)
-	    {
-	      error (0, 0, _("extra operand %s"), quote (argv[optind]));
-	      usage (EXIT_FAILURE);
-	    }
-	  file[nfiles++] = argv[optind++];
-	}
+          || (posixly_correct && nfiles != 0)
+          || ((optc = getopt_long (argc, argv,
+                                   "-0123456789Dcdf:is:uw:z", longopts, NULL))
+              == -1))
+        {
+          if (argc <= optind)
+            break;
+          if (nfiles == 2)
+            {
+              error (0, 0, _("extra operand %s"), quote (argv[optind]));
+              usage (EXIT_FAILURE);
+            }
+          file[nfiles++] = argv[optind++];
+        }
       else switch (optc)
-	{
-	case 1:
-	  {
-	    unsigned long int size;
-	    if (optarg[0] == '+'
-		&& posix2_version () < 200112
-		&& xstrtoul (optarg, NULL, 10, &size, "") == LONGINT_OK
-		&& size <= SIZE_MAX)
-	      skip_chars = size;
-	    else if (nfiles == 2)
-	      {
-		error (0, 0, _("extra operand %s"), quote (optarg));
-		usage (EXIT_FAILURE);
-	      }
-	    else
-	      file[nfiles++] = optarg;
-	  }
-	  break;
+        {
+        case 1:
+          {
+            unsigned long int size;
+            if (optarg[0] == '+'
+                && posix2_version () < 200112
+                && xstrtoul (optarg, NULL, 10, &size, "") == LONGINT_OK
+                && size <= SIZE_MAX)
+              skip_chars = size;
+            else if (nfiles == 2)
+              {
+                error (0, 0, _("extra operand %s"), quote (optarg));
+                usage (EXIT_FAILURE);
+              }
+            else
+              file[nfiles++] = optarg;
+          }
+          break;
 
-	case '0':
-	case '1':
-	case '2':
-	case '3':
-	case '4':
-	case '5':
-	case '6':
-	case '7':
-	case '8':
-	case '9':
-	  {
-	    if (skip_field_option_type == SFO_NEW)
-	      skip_fields = 0;
+        case '0':
+        case '1':
+        case '2':
+        case '3':
+        case '4':
+        case '5':
+        case '6':
+        case '7':
+        case '8':
+        case '9':
+          {
+            if (skip_field_option_type == SFO_NEW)
+              skip_fields = 0;
 
-	    if (!DECIMAL_DIGIT_ACCUMULATE (skip_fields, optc - '0', size_t))
-	      skip_fields = SIZE_MAX;
+            if (!DECIMAL_DIGIT_ACCUMULATE (skip_fields, optc - '0', size_t))
+              skip_fields = SIZE_MAX;
 
-	    skip_field_option_type = SFO_OBSOLETE;
-	  }
-	  break;
+            skip_field_option_type = SFO_OBSOLETE;
+          }
+          break;
 
-	case 'c':
-	  countmode = count_occurrences;
-	  break;
+        case 'c':
+          countmode = count_occurrences;
+          break;
 
-	case 'd':
-	  output_unique = false;
-	  break;
+        case 'd':
+          output_unique = false;
+          break;
 
-	case 'D':
-	  output_unique = false;
-	  output_later_repeated = true;
-	  if (optarg == NULL)
-	    delimit_groups = DM_NONE;
-	  else
-	    delimit_groups = XARGMATCH ("--all-repeated", optarg,
-					delimit_method_string,
-					delimit_method_map);
-	  break;
+        case 'D':
+          output_unique = false;
+          output_later_repeated = true;
+          if (optarg == NULL)
+            delimit_groups = DM_NONE;
+          else
+            delimit_groups = XARGMATCH ("--all-repeated", optarg,
+                                        delimit_method_string,
+                                        delimit_method_map);
+          break;
 
-	case 'f':
-	  skip_field_option_type = SFO_NEW;
-	  skip_fields = size_opt (optarg,
-				  N_("invalid number of fields to skip"));
-	  break;
+        case 'f':
+          skip_field_option_type = SFO_NEW;
+          skip_fields = size_opt (optarg,
+                                  N_("invalid number of fields to skip"));
+          break;
 
-	case 'i':
-	  ignore_case = true;
-	  break;
+        case 'i':
+          ignore_case = true;
+          break;
 
-	case 's':
-	  skip_chars = size_opt (optarg,
-				 N_("invalid number of bytes to skip"));
-	  break;
+        case 's':
+          skip_chars = size_opt (optarg,
+                                 N_("invalid number of bytes to skip"));
+          break;
 
-	case 'u':
-	  output_first_repeated = false;
-	  break;
+        case 'u':
+          output_first_repeated = false;
+          break;
 
-	case 'w':
-	  check_chars = size_opt (optarg,
-				  N_("invalid number of bytes to compare"));
-	  break;
+        case 'w':
+          check_chars = size_opt (optarg,
+                                  N_("invalid number of bytes to compare"));
+          break;
 
-	case 'z':
-	  delimiter = '\0';
-	  break;
+        case 'z':
+          delimiter = '\0';
+          break;
 
-	case_GETOPT_HELP_CHAR;
+        case_GETOPT_HELP_CHAR;
 
-	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+        case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 
-	default:
-	  usage (EXIT_FAILURE);
-	}
+        default:
+          usage (EXIT_FAILURE);
+        }
     }
 
   if (countmode == count_occurrences && output_later_repeated)
     {
       error (0, 0,
-	   _("printing all duplicated lines and repeat counts is meaningless"));
+           _("printing all duplicated lines and repeat counts is meaningless"));
       usage (EXIT_FAILURE);
     }
 
